/**
 * @file conveyorbelt.cpp
 * Implementation of ConveyorBelt class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "conveyorbelt.h"
#include "model.h"

#include <gfx/gl.h>

ConveyorBelt::ConveyorBelt( Model *model, Shape shape, Vec3 position, dMatrix3 orientation, Vec3 scale, double mass, bool on, bool stationary, bool movable, const bool rotatable[3] ) :
    ShapeObject( model, shape, position, orientation, scale, mass, on, stationary, movable, rotatable )
{
    _texture_slide = 0.0;
}

void ConveyorBelt::tick( double t )
{
    if ( is_on() ) {
        _texture_slide = fmod( _texture_slide + t , 1.0 );
    }
}

void ConveyorBelt::render( bool selected )
{
    glMatrixMode( GL_TEXTURE );
    glPushMatrix();

    glTranslated( 0.0, _texture_slide, 0.0 );

    glMatrixMode( GL_MODELVIEW );
    Object::render();

    glMatrixMode( GL_TEXTURE );
    glPopMatrix();

    glMatrixMode( GL_MODELVIEW );
}

void ConveyorBelt::collide( dContact* contact )
{
    Object::collide( contact );

    if ( !is_on() ) {
        // only push objects if we're on.
        return;
    }

    // find the tangent in the direction of motion
    Mat4 rot;
    dMatrix3ToMat4( rot, dBodyGetRotation( _body_id ) );

    Vec4 fdir1_4 = Vec4( get_forward_vec(), 0 );
    Vec3 fdir1 = vec_dropw( fdir1_4 );
    unitize( fdir1 );

    // set the surface in motion
    contact->surface.mode |= dContactFDir1 | dContactMotion1;
    contact->surface.motion1 = 75.0f;

    Vec3TodVector3( contact->fdir1, fdir1 );
}

