/**
 * @file actors/shapeobject.cpp
 * Implementation of ShapeObject class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include "shapeobject.h"
#include "model.h"
#include "scene.h"

ShapeObject::ShapeObject( Model *model, Shape shape, Vec3 position, dMatrix3 orientation, Vec3 scale, double mass, bool on, bool stationary, bool movable, const bool rotatable[3] ) :
    Object( model, position, orientation, scale, mass, on, stationary, movable, rotatable ),
    _shape( shape )
{
    Vec3 size = model->get_max() - model->get_min();
    size = vec_mult( _scale, size );

    // create ODE shape
    switch ( _shape ) {
    case SHAPE_BOX:
        dMassSetBoxTotal( &_mass, mass, size[ 0 ], size[ 1 ], size[ 2 ] );
        _geom_id = dCreateBox( get_space_id(), size[ 0 ], size[ 1 ], size[ 2 ] );
        break;
    case SHAPE_SPHERE:
        {
            double radius = min( size[ 0 ], min( size[ 1 ], size[ 2 ] ) ) / 2.0;

            dMassSetSphereTotal( &_mass, mass, radius );
            _geom_id = dCreateSphere( get_space_id(), radius );
            break;
        }
    case SHAPE_CYLINDER:
        {
            double radius = min( size[ 0 ], min( size[ 1 ], size[ 2 ] ) ) / 2.0;
            double length = max( size[ 0 ], max( size[ 1 ], size[ 2 ] ) ) - 2.0 * radius;

            dMassSetCappedCylinderTotal( &_mass, mass, 3, radius, length );
            _geom_id = dCreateCCylinder( get_space_id(), radius, length );
            break;
        }
    }

    setup_geom();
}
