/**
 * @file trigger.cpp
 * Implementation for Trigger class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "trigger.h"
#include "model.h"

Trigger::Trigger( Model *model, Shape shape, Vec3 position, dMatrix3 orientation, Vec3 scale, double mass, bool on, bool stationary, bool movable, const bool rotatable[3] ) :
    ShapeObject( model, shape, position, orientation, scale, mass, on, stationary, movable, rotatable )
{
    _timer = 0.0;
    // FIXME: might want to be able to set the on/off materials in the
    // level file.
    _model->set_material( red );
}

void Trigger::stop_sim( )
{
    Object::stop_sim();

    _model->set_material( red );
}

void Trigger::tick( double t )
{
    if ( _timer > 0.0 ) {
        _timer -= t;
    }

    if ( _timer <= 0.0 ) {
        set_on( false );
        _model->set_material( red );
        get_connection()->set_on( false );
    }
}

void Trigger::collide( dContact* contact )
{
    Object::collide( contact );

    _model->set_material( green );
    set_on( true );
    get_connection()->set_on( true );

    _timer = 5.0;
}

string Trigger::get_info( ) const
{
    return "connectionID=\"" + to_string( get_connection()->get_id() ) + "\" " +
        ShapeObject::get_info();
}

