/**
 * @file sample.h
 * Brief description of sample.h.
 *
 * Detailed description of sample.h.
 *
 * Our complete coding standard is at
 * http://www.acm.uiuc.edu/gamebuilders/services/projects/CodingStyle.txt
 *
 * @see sample.cpp for more info on this commenting thing.
 */
// Notice the format of the following header guard.  All headers should
// have guards like the following (_FILENAME_H)
#ifndef _SAMPLE_H
#define _SAMPLE_H

// Use the std C++ headers (without the .h)
#include <vector>

// I'm lazy.  Is anyone else?
using namespace std;

char *sample_func(char x, int y);

/**
 * Sample Class.
 * 
 * This will serve to demonstrate our class conventions.  Notice the order
 * of access specifiers: public, protected, private.  The more likely you
 * are to need it, the higher up it should be.  Also for this reason,
 * withing each access group, things should be declared:
 * typedefs/enums/classes, static methods, methods, static variables,
 * variables.
 *
 * Name classes and other types by CapitalizingEachWord, with no
 * underscores.  Name functions and variables in
 * lower_case_with_underscores.  Private stuff is prefixed with an _.
 * Notice how access specifiers are of the same indent level as the "class"
 * keyword.  Each indent is 4 spaces.
 */
class Sample {
public:
    static void static_func();

    Sample();
    int demonstrate_style(int n);
protected:
    // nothing.. just for demonstration.
private:
    /**
     * A triplet of ints.
     *
     * This is our data item for what Sample holds.
     */
    struct Triple {
        int one; ///< first
        int two; /**< second */
        int three; /**< third */
    };
    typedef vector<Triple> Container; /**< we'll use a vector to hold our Triples */

    int _private_func(int x, int y);

    int _super_private_var; /**< doc for a variable (the < means item precedes comment) */
    Container _contents; /**< my container of Triples */
};

// Notice the end of the header guard.  Please comment it like so.  Don't
// put the name of the file there, because that's just one more thing to
// change if the filename changes.
#endif // header guard
