/**
 * @file behavioreffect.h
 * Header for BehaviorEffect class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _BEHAVIOREFFECT_H
#define _BEHAVIOREFFECT_H

#include "effects.h"

/**
 * Governs the behaviors of animals.
 *
 * Behavior effects can have magnitudes, which signals to the receiver how
 * big this effect is.  This has different meanings depending on the type
 * of effect: for FearEffects, magnitude is how scary the monster is; for
 * HungerEffects, magnitude is how tasty the food is; and so on.
 */
class BehaviorEffect : public Effect {
public:
    /**
     * Dummy constructor, pretty much here solely for adding to the
     * receiver list.
     */
    BehaviorEffect() { }

    /**
     * Constructor for a Behavior effect.
     * @param magnitude how scary the monster, how tasty the food, etc
     */
    BehaviorEffect(Actor *sender, int magnitude) :
        Effect( sender ), _magnitude(magnitude) { }

    /**
     * @return the intensity of the behavior effect.
     */
    int get_magnitude()
        { return _magnitude; }

    virtual RecList& get_receiver_list()
        { return _receiver_list; }
protected:
    int _magnitude;  ///< how scary the monster, how tasty the food, etc.
private:
    static RecList _receiver_list;
};

#define TO_BEHAVIOR_EFFECT(pobj)    TO_EFFECT(pobj, BehaviorEffect)
#define IS_BEHAVIOR_EFFECT(pobj)    TO_BEHAVIOR_EFFECT(pobj)

#endif // header guard
