/**
 * @file powereffect.h
 * Header for PowerEffect class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _POWEREFFECT_H
#define _POWEREFFECT_H

#include "effects.h"

/**
 * Delivered to a receiver of some power, like electricity.
 *
 * Usually used to turn something on, like a lightbulb, or a generator,
 * etc.
 */
class PowerEffect : public Effect {
public:
    /**
     * Dummy constructor, pretty much here solely for adding to the
     * receiver list.
     */
    PowerEffect() { }

    /**
     * Constructor for a Power effect.
     * @param magnitude the intensity of the power being supplied.
     */
    PowerEffect(Actor *sender, int magnitude) : Effect(sender), _magnitude( magnitude ) { }

    /**
     * @return the intensity of the power effect.
     */
    int get_magnitude()
        { return _magnitude; }

    virtual RecList& get_receiver_list()
        { return _receiver_list; }
protected:
    int _magnitude;  ///< how bright the light, how fast the motor, etc.
private:
    static RecList _receiver_list;
};

#define TO_POWER_EFFECT(pobj)    TO_EFFECT(pobj, PowerEffect)
#define IS_POWER_EFFECT(pobj)    TO_POWER_EFFECT(pobj)

#endif // header guard
