/**
 * @file /home/guy/src/AP/gamestate/level.h
 * Header file for /home/guy/src/AP/gamestate/level class.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _LEVEL_H
#define _LEVEL_H

#include <ClanLib/core.h>

#include <map>
#include <string>

#include "main.h"
#include "object.h"
#include "actor.h"
#include "goal.h"
#include "worldbox.h"

class Level {
public:

    typedef map<int, Object*> ObjectList;
    typedef map<int, Actor*> ActorList;
    typedef map<int, Goal*> GoalList;
    typedef ObjectList::iterator ObjectIterator;
    typedef ActorList::iterator ActorIterator;
    typedef GoalList::iterator GoalIterator;

    Level( string filename );

    ~Level( void );

    Object *new_object_from_dom( const CL_DomElement& element );

    const bool level_completed( void );

    ObjectList& get_objects( void ) { return _objects; }
    ActorList& get_actors( void ) { return _actors; }
    int get_level_id( void ) const { return _id; }
    string get_filename( void ) const { return _filename; }
    string get_description( void ) const { return _description; }
    Vec3 get_world_size( ) const { return _world_box->get_size(); }

    void tick( double t );

    void render( void );

    void stop_sim( void );

    void reset_level( void );

    void save( string filename = "" );
    void save( ostream& out );
public:

    int _id;

    string _filename;

    string _description;

    GoalList _goals;
    ObjectList _objects;
    ActorList _actors;

    WorldBox* _world_box;

    double _camera_radius;
    double _camera_elevation;
    double _camera_azimuth;
};

#endif // header guard
