/**
 * @file main.cpp
 * Implementation of main application.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include <ClanLib/core.h>
#include <ClanLib/gui.h>
#include <ClanLib/gl.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/sound.h>

#include <iostream>
using namespace std;

#include "gamestate.h"
#include "gui.h"
#include "scene.h"

#define CL_MODULES( func )   \
    CL_SetupCore::func();    \
    CL_SetupSound::func();   \
    CL_SetupDisplay::func(); \
    CL_SetupGL::func();      \
    CL_SetupGUI::func()

class AP : public CL_ClanApplication {
public:
    virtual int main(int argc, char** argv)
    {
        // Create a console window for text-output if not available
        CL_ConsoleWindow console("Console", 80, 10000 );
        console.redirect_stdio();

        try {
            cout << "Initializing ClanLib..." << endl;

            CL_MODULES( init );

            CL_SoundOutput output( 44100 );
            output.set_global_volume( 100 );

            cout << "Creating GameState..." << endl;

            game_state = new GameState();

            cout << "Creating GUI..." << endl;

            gui = new GUI();

            cout << "Adding Resources..." << endl;

            if( gui->add_resources( "resources/gui/GUIStyleSilver/gui.xml" ) < 0 )
                return -1;

            if( gui->add_resources( "resources/gui/gui.xml" ) < 0 )
                return -1;

            cout << "Initializing GUI..." << endl;

            if( gui->initialize( game_state ) < 0 )
                return -1;

            cout << "Registering GUI signals..." << endl;

            if( gui->register_signals() < 0 )
                return -1;

            cout << "Creating SceneManager..." << endl;

            scene_manager = new SceneManager( gui->get_gc(), game_state );

            cout << "Registering SceneManager signals..." << endl;

            if( scene_manager->register_signals() < 0 )
                return -1;

            cout << "Entering Game Loop..." << endl;

            last_tick = CL_System::get_time();

            while( !gui->quit() )
            {
                cur_tick = CL_System::get_time();
//                double t = double(cur_tick - last_tick) / 1000.0; 

                game_state->tick( 0.01 );

                scene_manager->tick( 0.01 );

                last_tick = cur_tick;

                scene_manager->render();

                gui->render();

                CL_Display::flip();
                CL_System::keep_alive( 10 );
            }

            gui->finish();

            dCloseODE();

            CL_MODULES( deinit );
        }
        catch ( CL_Error e ) {
            cout << e.message << endl;

            // Display console close message and wait for a key
            console.display_close_message();
        }

        return 0;
    }

private:

    GameState* game_state;

    GUI* gui;

    SceneManager* scene_manager;

    int last_tick, cur_tick;

} app;
