#include "odeutils.h"

void dMatrix3ToOpenGL( float oglMat[ 16 ], const dMatrix3 dMat3 )
{
    oglMat[  0 ] = dMat3[  0 ];
    oglMat[  1 ] = dMat3[  4 ];
    oglMat[  2 ] = dMat3[  8 ];
    oglMat[  3 ] = 0.0f; 

    oglMat[  4 ] = dMat3[  1 ];
    oglMat[  5 ] = dMat3[  5 ];
    oglMat[  6 ] = dMat3[  9 ];
    oglMat[  7 ] = 0.0f;

    oglMat[  8 ] = dMat3[  2 ];
    oglMat[  9 ] = dMat3[  6 ];
    oglMat[ 10 ] = dMat3[ 10 ];
    oglMat[ 11 ] = 0.0f;

    oglMat[ 12 ] = 0.0f;
    oglMat[ 13 ] = 0.0f;
    oglMat[ 14 ] = 0.0f;
    oglMat[ 15 ] = 1.0f;
}

void Mat4TodMatrix3( dMatrix3 dmat3, const Mat4 mat4 )
{
    dmat3[  0 ] = mat4( 0, 0 );
    dmat3[  1 ] = mat4( 0, 1 );
    dmat3[  2 ] = mat4( 0, 2 );
    dmat3[  3 ] = mat4( 0, 3 );
                            
    dmat3[  4 ] = mat4( 1, 0 );
    dmat3[  5 ] = mat4( 1, 1 );
    dmat3[  6 ] = mat4( 1, 2 );
    dmat3[  7 ] = mat4( 1, 3 );
                            
    dmat3[  8 ] = mat4( 2, 0 );
    dmat3[  9 ] = mat4( 2, 1 );
    dmat3[ 10 ] = mat4( 2, 2 );
    dmat3[ 11 ] = mat4( 2, 3 );
}

void dMatrix3ToMat4( Mat4& mat4, const dMatrix3 dmat3 )
{
    mat4( 0, 0 ) = dmat3[  0 ];
    mat4( 0, 1 ) = dmat3[  1 ];
    mat4( 0, 2 ) = dmat3[  2 ];
    mat4( 0, 3 ) = 0.0;

    mat4( 1, 0 ) = dmat3[  4 ];
    mat4( 1, 1 ) = dmat3[  5 ];
    mat4( 1, 2 ) = dmat3[  6 ];
    mat4( 1, 3 ) = 0.0;

    mat4( 2, 0 ) = dmat3[  8 ];
    mat4( 2, 1 ) = dmat3[  9 ];
    mat4( 2, 2 ) = dmat3[ 10 ];
    mat4( 2, 3 ) = 0.0;

    mat4( 3, 0 ) = dmat3[ 12 ];
    mat4( 3, 1 ) = dmat3[ 13 ];
    mat4( 3, 2 ) = dmat3[ 14 ];
    mat4( 3, 3 ) = 1.0;
}

void dMatrix3TodMatrix3( dMatrix3 dmat3_1, const dMatrix3 dmat3_2 )
{
    memcpy( dmat3_1, dmat3_2, sizeof( dMatrix3 ) );
}

void dRealPtrTodMatrix4( dMatrix4 dmat4, const float* fltPtr )
{
    dmat4[  0 ] = fltPtr[  0 ];
    dmat4[  1 ] = fltPtr[  1 ];
    dmat4[  2 ] = fltPtr[  2 ];
    dmat4[  3 ] = 0.0f;

    dmat4[  4 ] = fltPtr[  3 ];
    dmat4[  5 ] = fltPtr[  4 ];
    dmat4[  6 ] = fltPtr[  5 ];
    dmat4[  7 ] = 0.0f;

    dmat4[  8 ] = fltPtr[  6 ];
    dmat4[  9 ] = fltPtr[  7 ];
    dmat4[ 10 ] = fltPtr[  8 ];
    dmat4[ 11 ] = 0.0f;

    dmat4[ 12 ] = fltPtr[  9 ];
    dmat4[ 13 ] = fltPtr[ 10 ];
    dmat4[ 14 ] = fltPtr[ 11 ];
    dmat4[ 15 ] = 0.0f;
}

void dRealPtrTodMatrix3( dMatrix3 dmat3, const float* fltPtr )
{
    dmat3[  0 ] = fltPtr[  0 ];
    dmat3[  1 ] = fltPtr[  1 ];
    dmat3[  2 ] = fltPtr[  2 ];
    dmat3[  3 ] = fltPtr[  3 ];
    dmat3[  4 ] = fltPtr[  4 ];
    dmat3[  5 ] = fltPtr[  5 ];
    dmat3[  6 ] = fltPtr[  6 ];
    dmat3[  7 ] = fltPtr[  7 ];
    dmat3[  8 ] = fltPtr[  8 ];
    dmat3[  9 ] = fltPtr[  9 ];
    dmat3[ 10 ] = fltPtr[ 10 ];
    dmat3[ 11 ] = fltPtr[ 11 ];
}

#if 0
void dRealPtrTodQuaternion( dQuaternion dquat, const float* fltPtr )
{
    dquat[ 0 ] = fltPtr[ 0 ];
    dquat[ 1 ] = fltPtr[ 1 ];
    dquat[ 2 ] = fltPtr[ 2 ];
    dquat[ 3 ] = fltPtr[ 3 ];
}

void dRealPtrToVec3( Vec3& vec3, const float* fltPtr )
{
    vec3[ 0 ] = fltPtr[ 0 ];
    vec3[ 1 ] = fltPtr[ 1 ];
    vec3[ 2 ] = fltPtr[ 2 ];
}
#endif
