/**
 * @file /home/guy/src/AP/scene/material.h
 * Header file for /home/guy/src/AP/scene/material class.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _MATERIAL_H
#define _MATERIAL_H

#include "main.h"

#include <gfx/vec4.h>
#include <ClanLib/gl.h>

#include <string>
using namespace std;

class Material {
public:
    Vec4f ambient;    ///< Fraction of ambient light reflected
    Vec4f diffuse;    ///< Fraction of incoming light reflected diffusely 
    Vec4f specular;   ///< Fraction of incoming light reflected specularly
    Vec4f emissive;   ///< Light emitted by this material

    double shininess;   ///< Exponent for Phong illumination model

    CL_OpenGLSurface* texture;     ///< OpenGL texture index
    string texture_filename;    ///< Filename of texture, for reloading

    Material() : texture(NULL) {}
    Material(Vec4f amb, Vec4f diff, Vec4f spec, Vec4f emit, double shiny);
    Material(const Material& mat);

    const Material& operator=(const Material& mat);

    ~Material();

    Material copy_with_texture( string filename ) const;

    void reload_texture();

    void make_current() const;
};

CL_OpenGLSurface* load_gl_texture( string filename );

extern Material red;
extern Material green;
extern Material blue;
extern Material grey;

#endif // header guard
