/**
 * @file model.h
 * Header for Model class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _MODEL_H
#define _MODEL_H

#include "utils.h"
#include "material.h"

/**
 * Draw a wireframe axis-aligned box whose opposite corners are given by the
 * points 'min' and 'max', in the color 'color'.
 */
void draw_wire_cube( const Vec3& min, const Vec3& max, const Vec3f& color );

/**
 * Draw a solid axis-aligned box whose opposite corners are given by the
 * points 'min' and 'max'.  Only draw the faces specified by draw_faces.
 */
void draw_solid_cube( const Vec3& min, const Vec3& max, const bool draw_faces[6] = NULL );

/// Draw a capped cylinder aligned along the z axis
void draw_capped_cylinder( double length, double radius, int quality = 3 );

#define CUBE_FRONT  0
#define CUBE_BACK   1
#define CUBE_LEFT   2
#define CUBE_RIGHT  3
#define CUBE_TOP    4
#define CUBE_BOTTOM 5


/**
 * A graphical representation of an object.
 */
class Model {
public:
    Model( void );
    Model( const Material& mat );

    virtual ~Model( void ) {}

    /**
     * Render the object to the screen.
     */
    virtual void render( void );

    /**
     * Draw selection decoration to let the user know we're picked
     */
    virtual void draw_decor( Vec3 color )
    { draw_wire_cube( get_min()*1.02, get_max()*1.02, color ); }

    virtual Vec3 get_min( void ) { return Vec3( -1.0 ); }
    virtual Vec3 get_max( void ) { return Vec3( 1.0 ); }

    virtual string get_info()
        { return "texture=\"" + _material.texture_filename + "\""; }

    void set_material( Material m ) { _material = m; _material.reload_texture(); }
private:
    Material _material; ///< the OpenGL material for this class.
};

#endif // header guard
