/**
 * @file /home/guy/src/AP/scene/scene.h
 * Header file for /home/guy/src/AP/scene/scene class.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _SCENE_H
#define _SCENE_H

#include <ClanLib/core.h>
#include <ClanLib/gl.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>

#include <ode/ode.h>

#include "main.h"
#include "object.h"
#include "camera.h"
#include "gamestate.h"

#include <map>
using namespace std;

// since everyone else needs these, need to keep the variables static and the
// associated accessors global
dWorldID get_world_id( void );
dSpaceID get_space_id( void );

class SceneManager {
public:
    static SceneManager* get_scene() { return _scene; }

    SceneManager( CL_GraphicContext* gc, GameState* gamestate );
    ~SceneManager( void );

    int register_signals( void );

    /**
     * Elapse t seconds.
     *
     * This calls our our actor's tick functions, then updates all objects'
     * positions.
     */
    void tick( double t );

    /**
     * Render the scene.
     */
    void render( void );

    /**
     * Draw axes projecting from obj.
     */
    void draw_axes( Object* obj );

    /**
     * Select the object at screen position pos.
     * _selected_obj contains the picked object, or NULL if background was
     * clicked.
     * @param pos the x,y window coordinates that were selected.
     */
    void select_object( int *pos );

    /**
     * Deselect the currently selected object.
     */
    void deselect_object( void );

    Object *get_selected_object( void ) { return _selected_obj; }

    Camera *get_camera() { return &_camera; }

    // HACK, see gui's on_mouse_down
    void on_mouse_down( const CL_InputEvent& event );

    bool is_editor_mode() { return _editor_mode; }

    void begin_level( void );
    void finish_level( void );

private:

    void generate_handlers( void );
    
    void on_mouse_move( const CL_InputEvent& event );
    void on_key_down( const CL_InputEvent& event );
    void on_key_up( const CL_InputEvent& event );

    static void near_callback( void* data, dGeomID o1, dGeomID o2 );

    void _check_selected_colliding();

    /// Update the mouse position based on where the selected object is.
    void _update_mouse_pos();

    static SceneManager *_scene;

    GameState* _gamestate;

    CL_Slot _slot_click;
    CL_Slot _slot_move;
    CL_Slot _slot_keydown;
    CL_Slot _slot_keyup;

    void gl_init( void );
  
    CL_OpenGLState* _glstate;

    Object *_selected_obj;      ///< currently selected object

    Camera _camera;             ///< our camera orientation and position

    int _ignore_move_signal;    ///< ignore this many move signals

    int _prev_mouse_pos[ 2 ];
    
    int _current_level_id;

    bool _editor_mode;

    bool _show_movables;    // show movable objects in blue

    bool _selected_colliding; // is the selected object colliding

    enum AxesType {
        DRAW_ON_NONE,       // don't draw axes
        DRAW_ON_SELECTED,   // draw only on selected object
        DRAW_ON_ALL,        // draw on all objects
        DRAW_NUM_TYPES     // number of types
    } _axes_type;
};

#endif // header guard
